# Hi! this script is a awk program script.
# Usage is: time awk -f parse1.awk usbsnoop.log > log1
#
# What does it do ?
# Extract usefull infos from sniffusb logs
#
# It works with sniffusb 1.8
#
# Loosely based on a script by Thomas SOUMARMON (soumarmt@hotmail.com).


BEGIN {

	action = ""; 

	buffer[0] = 0;

	rtype = 0;
	request = 0;
	value = 0;
	rindex = 0;

	ignore_urb = 0;

	last_int_urb = -1;
}

{
	if(NR==1)
		urb = strtonum($5)-1;

	if ($4 == "URB") {

# New URB
		this_urb = strtonum($5);

		if (this_urb == last_int_urb) {
# this interrupt URB is coming back
			ignore_urb = 1;
			next;
		}

		if (way == "down" && $7 == "down") {
# This URB is also an interrupt URB
			printf "URB %5d  skip 1\n", this_urb;
			ignore_urb = 1;
			last_int_urb = this_urb;
			next;
		}
		
		old_way = way;
		way = $7;

		if (way == "back") {
			if (urb != this_urb) {
				printf "got URB back %d instead of %d at line %d\n", this_urb, urb, NR;
				#exit;
			}
		}
		else if (way == "down") {
			
			ignore_urb = 0;

			if (this_urb != urb+1 && this_urb != urb+2) {
				printf "expected URB down %d instead of %d at line %d\n", urb+1, this_urb, NR;
				#exit;
			}

			if (last_int_urb != urb) {

				if (endpoint == "CONTROL") {
					printf "URB %5d  control  0x%02x 0x%02x 0x%02x 0x%02x len %5d", urb, rtype, request, value, rindex, buflen;
					if (direction == "OUT") {
						printf(" wrote ");
					} else {
						printf(" read  ");
					}

					for (i=0; i<buflen; i++)
						printf("0x%02x ", buffer[i]);
					printf("\n");
				}
				else if (endpoint == "BULK_IN") {
					printf "URB %5d  bulk_in  len %5d  read  ", urb, buflen;

					for (i=0; i<buflen; i++)
						printf("0x%02x ", buffer[i]);
					printf("\n");
				}
				else if (endpoint == "BULK_OUT") {
					printf "URB %5d  bulk_out len %5d  wrote ", urb, buflen;
				
					for (i=0; i<buflen; i++)
						printf("0x%02x ", buffer[i]);
					printf("\n");
				}
			}
		}
		else {
			printf("BAD line %d", NR);
			exit;
		}


		/* Next urb */
			 urb = strtonum($5);
	}

	if (ignore_urb) {
		next;
	}

	if ($1 == "PipeHandle") {
		if ($5 == "0x00000002]") endpoint = "BULK_OUT";
		else if ($5 == "0x00000081]") endpoint = "BULK_IN";
		else if ($5 == "0x00000083]") {
			printf "URB %5d  skip 2\n", urb;
			ignore_urb = 1;
			way = "back";		# do as if we got the answer
			last_int_urb = urb;
			next;
		}
		else endpoint = "CONTROL";
	}

	if ($1 == "TransferBufferLength") buflen=strtonum("0x"$3);
	if ($1 == "RequestTypeReservedBits") rtype=strtonum("0x"$3);
	if ($1 == "Request") request=strtonum("0x"$3);
	if ($1 == "Value") value=strtonum("0x"$3);
	if ($1 == "Index") {
		rindex=strtonum("0x"$3);
		get_data = 0;
	}

	if ($4 == "(USBD_TRANSFER_DIRECTION_IN,") direction="IN";
	if ($4 == "(USBD_TRANSFER_DIRECTION_OUT,") direction="OUT";

	if ($2 == "URB_FUNCTION_VENDOR_DEVICE:") action="vendor";
	if ($2 == "URB_FUNCTION_BULK_OR_INTERRUPT_TRANSFER:") action="bulk";
	if ($2 == "URB_FUNCTION_CONTROL_TRANSFER:") action="control";

# Take care of the data
	if ($1 == "UrbLink") {
		get_data = 0;
		if (data_left != 0) {
			printf("error line %d (data left)\n", NR);
			exit;
		}
	}

	if (get_data) {
		if (data_left > 16)
			data_len = 16;
		else
			data_len = data_left;

		if (NF == (data_len+1)) {
			data_left = data_left - data_len;

			for (i=0; i<data_len; i++) {
				buffer[data_index] = strtonum("0x"$(i+2));
				data_index ++;
			}
		} else {
			printf("error line %d (invalid data)\n", NR);
			exit;
		}
	}

# We don't want to get data all the time.
	if ($1 == "TransferBufferMDL") {
		if ((direction == "OUT" && way == "down") ||
			(direction == "IN" && way == "back")) {
			
			get_data = 1;
			data_left = buflen;
			data_index = 0;
		}
	}
}


